unit IRQThreads;

interface

uses
  Classes, Windows, MainUnit;

type
  TIRQThread = class(TThread)
  public
    COSPPIs, COSPPISize: LongWord;
    CardNum: Integer;
  protected
    COSData: TCOSData;
    LastError: LongWord;

    procedure ReportIRQ;
    procedure ReportError;

    procedure Execute; override;
  end;

implementation

uses
  SysUtils, AIOWDM;

procedure TIRQThread.Execute;
begin
  SetLength(COSData, COSPPIs * COSPPISize);
  while COSWaitForIRQ(CardNum, COSPPIs, @COSData[0]) <> 0 do
    Synchronize(ReportIRQ)
  ;
  LastError := GetLastError;
  Synchronize(ReportError);
end;

procedure TIRQThread.ReportError;
begin
  if LastError = ERROR_OPERATION_ABORTED then
    MainForm.SetStatus('IRQ request aborted.')
  else if LastError = ERROR_INVALID_FUNCTION then
    MainForm.SetStatus('Error: this card already has a pending IRQ request. IRQ request aborted.')
  else
    MainForm.SetStatus('Error ' + IntToStr(LastError) + ' occurred in the driver. IRQ request aborted.')
  ;
end;

procedure TIRQThread.ReportIRQ;
begin
  MainForm.ReceiveCOSData(COSPPIs, COSPPISize, COSData);
end;

end.
