unit MainUnit;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ExtCtrls, ComCtrls;

type
  TCOSData = array of Byte;

  TMainForm = class(TForm)
    Panel1: TPanel;
    Label1: TLabel;
    CardCombo: TComboBox;
    SchemeRadios: TRadioGroup;
    CountRadios: TRadioGroup;
    Panel2: TPanel;
    GoButton: TButton;
    StopButton: TButton;
    PPILabel0: TLabel;
    PPILabel1: TLabel;
    PPILabel2: TLabel;
    MainStatus: TStatusBar;
    procedure FormCreate(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure GoButtonClick(Sender: TObject);
    procedure StopButtonClick(Sender: TObject);
  protected
    { Private declarations }
  public
    PPILabel: array[0..2] of TLabel;

    procedure SetStatus(NewStatus: String);
    procedure ReceiveCOSData(COSPPIs, COSPPISize: LongWord; const COSData: TCOSData);
    procedure ClearCOSData;
    { Public declarations }
  end;

var
  MainForm: TMainForm;

implementation

{$R *.DFM}

uses
  AIOWDM,
  IRQThreads;

var
  IRQThread: TIRQThread;

procedure TMainForm.FormCreate(Sender: TObject);
var
  iCard: Integer;
  CardStr: String;
  DeviceID, NomSize: LongWord;
  Nom: AnsiString;

begin
  PPILabel[0] := PPILabel0;
  PPILabel[1] := PPILabel1;
  PPILabel[2] := PPILabel2;

  for iCard := 0 to GetNumCards-1 do begin
    NomSize := 256;
    SetString(Nom, nil, NomSize);
    if QueryCardInfo(iCard, @DeviceID, nil, @NomSize, @Nom[1]) = 0 then begin
      CardStr := 'Error ' + IntToStr(GetLastError) + ' From QueryCardInfo';
    end
    else begin
      Dec(NomSize);
      SetLength(Nom, NomSize);

      CardStr := Nom + '(' + IntToHex(DeviceID, 4) + ')';
    end;

    CardStr := '[' + IntToStr(iCard) + '] ' + CardStr;
    CardCombo.Items.Add(CardStr);
  end;

  if CardCombo.Items.Count <> 0 then CardCombo.ItemIndex := 0;
end;

procedure TMainForm.FormClose(Sender: TObject; var Action: TCloseAction);
var
  iCard: Integer;

begin
  for iCard := 0 to CardCombo.Items.Count-1 do
    AbortRequest(iCard)
  ;
end;

procedure TMainForm.GoButtonClick(Sender: TObject);
var
  iCard: Integer;
  COSPPISize: LongWord;

begin
  GoButton.Enabled := False;

  CardCombo.Enabled := False;
  SchemeRadios.Enabled := False;
  CountRadios.Enabled := False;

  iCard := CardCombo.ItemIndex;

  case SchemeRadios.ItemIndex of
    0: begin
      COSPPISize := 3;
      if SetCOSScheme(iCard, COSScheme_24S) = 0 then begin
        SetStatus('Error ' + IntToStr(GetLastError) + ' from SetCOSScheme.');
        Exit;
      end;
    end;
    1: begin
      COSPPISize := 1;
      if SetCOSScheme(iCard, COSScheme_IIRO) = 0 then begin
        SetStatus('Error ' + IntToStr(GetLastError) + ' from SetCOSScheme.');
        Exit;
      end;
    end;
    else Exit;
  end;
  IRQThread := TIRQThread.Create(True);
  IRQThread.COSPPISize := COSPPISize;
  IRQThread.COSPPIs := CountRadios.ItemIndex + 1;
  IRQThread.CardNum := iCard;
  SetStatus('Detecting COS IRQs...');
  IRQThread.Resume;

  StopButton.Enabled := True;
  StopButton.SetFocus;
end;

procedure TMainForm.StopButtonClick(Sender: TObject);
begin
  StopButton.Enabled := False;

  SetStatus('Aborting IRQ request...');
  AbortRequest(CardCombo.ItemIndex);
  FreeAndNil(IRQThread);

  CardCombo.Enabled := True;
  SchemeRadios.Enabled := True;
  CountRadios.Enabled := True;

  GoButton.Enabled := True;
  GoButton.SetFocus;
end;

procedure TMainForm.SetStatus(NewStatus: String);
begin
  MainStatus.SimpleText := NewStatus;
end;

procedure TMainForm.ReceiveCOSData(COSPPIs, COSPPISize: LongWord; const COSData: TCOSData);
var
  iPPI: Integer;

begin
  case COSPPISize of
    3: for iPPI := 0 to COSPPIs-1 do begin
      PPILabel[iPPI].Caption := IntToHex(COSData[iPPI*3], 2) + ' ' + IntToHex(COSData[iPPI*3+1], 2) + ' ' + IntToHex(COSData[iPPI*3+2], 2);
    end;
    1: for iPPI := 0 to COSPPIs-1 do begin
      PPILabel[iPPI].Caption := IntToHex(COSData[iPPI], 2);
    end;
  end;
end;

procedure TMainForm.ClearCOSData;
var
  iPPI: Integer;

begin
  for iPPI := 0 to High(PPILabel) do
    PPILabel[iPPI].Caption := ''
  ;
end;

end.
